/*
 * Copyright (c) 2015-2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== watchdog.c ========
 */
#include <stdint.h>
#include <stddef.h>
#include <unistd.h>

/* Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/Power.h>
#include <ti/drivers/Watchdog.h>
#include <ti/drivers/pin/PINCC26XX.h>
#include <ti/drivers/gpio/GPIOCC26XX.h>
#include <ti/drivers/PIN.h>
   
   
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Event.h>
#include <ti/sysbios/knl/Mailbox.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/drivers/UART.h>
#include <ti/sysbios/BIOS.h>
#include <pthread.h>


/* Driver configuration */
#include "ti_drivers_config.h"

#define THREADSTACKSIZE    1024

#define TIMEOUT_MS      1000
#define SLEEP_US        500000

typedef void (*watchdogCallback_t)( void );
watchdogCallback_t g_cb;
PIN_Handle g_pinHandle[32];
PIN_State  g_pinState[32];
uint32_t   pin_enable_disable_array;


typedef enum
{
    WATCHDOG_DISABLE_RESET, //!< disable reset - debug mode
    WATCHDOG_ENABLE_RESET   //!< enable reset - regular mode
}watchdogReset_t;

Watchdog_Params wp;
Watchdog_Handle g_watchdogHandler;
void watchdogTimerCallback(void);


struct 
{
    Event_Struct strct;
    Event_Handle handle;
}event;

void *mainThread(void *arg0);


/*
 *  ======== board_openPin ========
 */
void board_openPin( PIN_Id pinId, PIN_Config config )
{

    PIN_Config pinConfig[2];
    pinConfig[0] = PIN_ID( pinId ) | config;
    pinConfig[1] = PIN_TERMINATE;
    g_pinHandle[pinId] = PIN_open( &g_pinState[pinId], pinConfig );
}


/*
 *  ======== board_setPin ========
 */
void board_setPin( PIN_Id pinId )
{
   PIN_Status ps = PIN_setOutputValue( g_pinHandle[pinId], pinId, 1 );
}


/*
 *  ======== board_clrPin ========
 */
void board_clrPin( PIN_Id pinId )
{
   PIN_Status ps = PIN_setOutputValue( g_pinHandle[pinId], pinId, 0 );
}


/*
 *  ======== watchdogDriver_initModule ========
 */
void watchdogDriver_initModule( watchdogReset_t resetEn, watchdogCallback_t cb )
{
    g_cb = cb;
    
    Watchdog_init();
    Watchdog_Params_init( &wp );
    wp.callbackFxn = (Watchdog_Callback) watchdogTimerCallback;
    
    wp.debugStallMode = Watchdog_DEBUG_STALL_ON;
    wp.resetMode      = Watchdog_RESET_OFF;

    g_watchdogHandler = Watchdog_open(CONFIG_WATCHDOG_0, &wp);
    Watchdog_setReload(g_watchdogHandler, 1500000); 
}


/*
 *  ======== watchdogTimerCallback ========
 */
void watchdogTimerCallback(void)
{
    // Set port 22
    board_setPin(GPIOCC26XX_DIO_22);
    
    //post Event to mainTread
    Event_post( event.handle, 0x01 );

    Watchdog_clear( g_watchdogHandler );

    // Set port 22
    board_clrPin(GPIOCC26XX_DIO_22);

}

/*
 *  ======== mainThread_init ========
 */
extern void maiThread_init(void)
{
    pthread_t           thread;
    pthread_attr_t      attrs;
    struct sched_param  priParam;
    int                 retc;

        /* Initialize the attributes structure with default values */
    pthread_attr_init(&attrs);

    /* Set priority, detach state, and stack size attributes */
    priParam.sched_priority = 1;
    retc = pthread_attr_setschedparam(&attrs, &priParam);
    retc |= pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
    retc |= pthread_attr_setstacksize(&attrs, THREADSTACKSIZE);
    if (retc != 0) {
        /* failed to set attributes */
        while (1) {}
    }

    retc = pthread_create(&thread, &attrs, mainThread, NULL);
    if (retc != 0) {
        /* pthread_create() failed */
        while (1) {}
    }

    Event_construct(&event.strct,NULL);
	event.handle = Event_handle(&event.strct);
}

/*
 *  ======== mainThread ========
 */
void *mainThread(void *arg0)
{
    Watchdog_init();

    board_openPin(GPIOCC26XX_DIO_22, PIN_GPIO_OUTPUT_EN  | PIN_PUSHPULL | PIN_GPIO_LOW);
    board_openPin(GPIOCC26XX_DIO_24, PIN_GPIO_OUTPUT_EN  | PIN_PUSHPULL | PIN_GPIO_LOW);
    
    /* Open a Watchdog driver instance */
    watchdogDriver_initModule(WATCHDOG_DISABLE_RESET, watchdogTimerCallback);

    while (1) 
    {
        uint32_t events = Event_pend(event.handle, Event_Id_NONE,  0xFFFFFFFF, BIOS_WAIT_FOREVER);
        Task_sleep(100);
        board_setPin(GPIOCC26XX_DIO_24);
        board_clrPin(GPIOCC26XX_DIO_24);
    }
}
